@@ -88,8 +88,10 @@ our @EXPORT = qw/silent
                  rawstr showstr showcontrols displaystr
                  show_white show_empty_string
                  fmt_codestring
+                 verif_warning
                  verif_no_internals_mentioned
                  insert_loc_in_evalstr verif_eval_err
+                 tdir
                  timed_run
                  mycheckeq_literal expect1 mycheck _mycheck_end
                  arrays_eq hash_subset
@@ -123,7 +125,8 @@ sub bug(@) { @_=("BUG FOUND:",@_); goto &Carp::confess }
 
 # Parse manual-testing args from @ARGV
 my @orig_ARGV = @ARGV;
-our ($debug, $verbose, $silent, $savepath, $nobail, $nonrandom, %dvs);
+our ($debug, $verbose, $silent, $savepath, $nobail, $nonrandom,
+     $workdir, %dvs);
 use Getopt::Long qw(GetOptions);
 Getopt::Long::Configure("pass_through");
 GetOptions(
@@ -133,6 +136,7 @@ GetOptions(
   "nobail"            => \$nobail,
   "n|nonrandom"       => \$nonrandom,
   "v|verbose"         => \$verbose,
+  "workdir=s"         => \$workdir,
 ) or die "bad args";
 Getopt::Long::Configure("default");
 say "> ARGV PASSED THROUGH: ",join(",",map{ "'${_}'" } @ARGV)
@@ -164,6 +168,25 @@ if ($nonrandom) {
   }
 }
 
+# Return a Path::Tiny object to a temporary directory, which may be
+# specified with the --workdir command-line arg.
+sub tdir() {
+  state $tdir;
+  if (! $tdir) {
+    if ($workdir) {
+      $workdir = path($workdir);
+      if ($workdir->exists) {
+        warn("# Removing old workdir ", qsh($workdir),"\n");
+        $workdir->remove_tree;
+      }
+      $tdir = $workdir->mkdir;
+    } else {
+      $tdir = Path::Tiny->tempdir();
+    }
+  };
+  $tdir
+}
+
 sub import {
   my $target = caller;
 
@@ -186,8 +209,12 @@ sub import {
   #  Do not inport 1- and 2- or 3- character upper-case names, which are
   #  likely to clash with user variables and/or spreadsheet column letters
   #  (when using Spreadsheet::Edit).
+  #
+  #  The :no-Test2 tag is used by subtest .pl scripts which must not
+  #  generate any Test2 events!
   unless (delete $tags{":no-Test2"}) {
-    require Test2::V0; # a huge collection of tools
+    require Test2::V0; # a huge collection of tools (n.b. version is in dist.ini)
+    Test2::V0->VERSION(1.302214);  # AutoPrereqs *does* see this!
     Test2::V0->import::into($target,
       -no_warnings => 1,
       (map{ "!$_" } "A".."AAZ")
@@ -752,6 +779,24 @@ sub mycheck($$@) {
   }
 }
 
+# USAGE: verif_warning { code... } qr/.../  [,"optional description"]
+#                                 ^ NO COMMA!
+sub verif_warning(&$) {
+  my ($code, $expected, $desc) = @_;
+  local $_;  # preserve $1 etc. for caller
+  my $got = "";
+  local $SIG{__WARN__} = sub { $got .= join("", @_); };
+  $code->();
+  unless ($got =~ qr/$expected/) {
+    my @caller = caller(0);
+    my $ln = $caller[2];
+    my $fn = $caller[1];
+    confess "expected warning did not occur at $fn line $ln\n",
+            "  (expected $expected)\n",
+            "  (got '${got}')\n";
+  }
+}
+
 sub verif_eval_err(;$) {  # MUST be called on same line as the 'eval'
   my ($msg_regex) = @_;
   my @caller = caller(0);
@@ -829,7 +874,7 @@ sub clean_capture_output($) {
 sub my_capture(&) {
   my ($out, $err, @results) = &capture($_[0]);
   $out = clean_capture_output($out);
-  $err = clean_capture_errput($err);
+  $err = clean_capture_output($err);
   confess "my_capture: Must be called in list context to receive both stdout & err"
     unless wantarray;
   return( $out, $err, @results );
